local Depth = require( 'Scripts.Depth-First' )

function love.load()	
	love.graphics.setBackgroundColor( 255, 255, 255 )
	love.math.noise( os.time() ) 
	Timer = .001
	Time = 0
	ShouldContinue = true
	
	Quads, Maze, QuadWidth, QuadHeight = LoadImageInformation()
	Width, Height = 80, 60
	
	-- print( Width, Height )
	ScreenWidth, ScreenHeight = love.graphics.getWidth(), love.graphics.getHeight()
	
	-- Mazes
	--- DepthMaze = Depth.New( Width, Height )
	StartX, StartY = love.math.random( Width ), love.math.random( Height )
	x, y, DepthMaze, VisitedCells, ReverseAmount = Depth.NewStep( Width, Height, StartX, StartY )
	
	-- References
	DepthReference = DecodeMazeData( DepthMaze )
	
	-- Canvases
	DepthCanvas = SetToCanvas( DepthReference, Quads, Maze, QuadWidth, QuadHeight )
end

function love.draw()
	love.graphics.push()
		love.graphics.scale( ScreenWidth / ( Width * QuadWidth ), ScreenHeight / ( Height * QuadHeight ) )				
		
		love.graphics.setColor( 255, 0, 0, 255 )
		if ShouldContinue then
			love.graphics.rectangle( 'fill', ( x - 1 ) * QuadWidth, ( y - 1 ) * QuadHeight, QuadWidth, QuadHeight )
		end
		love.graphics.setColor( 0, 255, 0, 255 )
		love.graphics.rectangle( 'fill', ( StartX - 1 ) * QuadWidth, ( StartY - 1 ) * QuadHeight, QuadWidth, QuadHeight )
		
		love.graphics.setColor( 255, 255, 255, 255 )
		love.graphics.setDefaultFilter( 'nearest', 'nearest' )
		love.graphics.draw( DepthCanvas, 0, 0 )
	love.graphics.pop()
end

function love.update( dt ) 
	Time = Time + dt
	if ShouldContinue and Time > Timer then
		for a = 1, 1 do
			if not DepthReference[y] or not DepthReference[y][x] then 
				-- TakeScreenShot() 
				ShouldContinue = false 
				return 
			end
			
			x, y, ReverseAmount = Depth.Step( x, y, DepthMaze, VisitedCells, ReverseAmount )
			if not DepthReference[y] or not DepthReference[y][x] then 
				-- TakeScreenShot() 
				ShouldContinue = false 
				return 
			end
			DepthReference = DecodeMazeData( DepthMaze )
		end
		
		DepthCanvas = SetToCanvas( DepthReference, Quads, Maze, QuadWidth, QuadHeight )
		Time = 0
	end
end

function love.keypressed( Key, IsRepeat )
	ShouldContinue = not ShouldContinue
	if not ShouldContinue then love.window.setTitle( 'Paused' ) 
	else love.window.setTitle( 'Maze Creation Algorithms' ) end
end

function love.keyreleased( Key, IsRepeat )
end

function TakeScreenShot()
	local Image = love.graphics.newScreenshot()
	love.filesystem.write( 'Maze.png', Image )
	Image:encode( 'Maze.png' )
end

function MakePO2( Dimension )
	local a = 1
	while a < Dimension do
		a = a * 2
	end
	return a
end

function SetToCanvas( Reference, Quads, Maze, QuadWidth, QuadHeight )
	local Canvas = love.graphics.newCanvas( MakePO2( ScreenWidth ), MakePO2( ScreenHeight ) )
	love.graphics.setCanvas( Canvas )
		Canvas:clear()
		
		for y = 1, #Reference do
			for x = 1, #Reference[y] do
				-- print( Reference[y][x], Quads[ Reference[y][x] ] )
				love.graphics.draw( Maze, Quads[ Reference[y][x] ], ( x - 1 ) * QuadWidth, ( y - 1 ) * QuadHeight )
			end
		end
	love.graphics.setCanvas()
	
	return Canvas
end

function DecodeMazeData( Maze )
	local Reference = {}
	
	for y = 1, #Maze do
		Reference[y] = {}
		for x = 1, #Maze[y] do
			local String = ''
			
			if not Maze[y][x].LeftWall then String = String .. 'L' end
			if not Maze[y][x].RightWall then String = String .. 'R' end
			if not Maze[y][x].TopWall then String = String .. 'U' end
			if not Maze[y][x].BottomWall then String = String .. 'D' end
			
			Reference[y][x] = String
		end
	end
	
	return Reference
end

function LoadImageInformation()
	local Quads = {}
	
	local Maze = love.graphics.newImage( 'Images/Maze.png' )
	local QuadWidth, QuadHeight = 10, 10
	local QuadInformation = {
		L = { 0, 70 }, 
		LR = { 10, 70 }, 
		LU = { 20, 70 }, 
		LD = { 30, 70 }, 
		LRU = { 40, 70 }, 
		LRD = { 50, 70 }, 
		LUD = { 0, 80 }, 
		LRUD = { 10, 80 }, 
		R = { 20, 80 }, 
		U = { 30, 80 }, 
		D = { 40, 80 },
		RD = { 50, 80 }, 
		UD = { 0, 90 }, 
		RU = { 10, 90 }, 
		RUD = { 20, 90 }, 
	}
	QuadInformation[''] = { 30, 90 }
	for Index, Quad in pairs( QuadInformation ) do
		Quads[Index] = love.graphics.newQuad( QuadInformation[Index][1], QuadInformation[Index][2], QuadWidth, QuadHeight, Maze:getWidth(), Maze:getHeight() )
	end
	
	return Quads, Maze, QuadWidth, QuadHeight
end